{ File   : SEQExt.dpr
  Author : Deniz Oezmen
  Created: 2007-01-21
  Changed: 2009-03-13
   
  The Last Express SEQ converter.
}
program SEQExt;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  StrUtils,
  FrameDec in 'graphics\FrameDec.pas',
  SaveTGA  in 'graphics\SaveTGA.pas';

const
  SEQFrameSize = 68;

type
  TSEQHeader = packed record
    NumFrames: Longword;
    Unknown  : Longword;
  end;
  PSEQHeader = ^TSEQHeader;

  TSEQFrame = packed record
  end;

function OpenInFile(var InFile: file; const InName: AnsiString): Boolean;
begin
  FileMode := fmOpenRead;
  AssignFile(InFile, InName);

  {$I-}
  Reset(InFile, 1);
  {$I+}
  Result := (IOResult = 0)
end;

procedure Process(var SEQFile: file; SEQName: AnsiString);
var
  i,
  BufIdx   : Longint;
  SEQBuffer: array of Byte;
  SEQHeader: PSEQHeader;
begin
  // Since SEQ files do not store size information for the frames, we are simply
  // going to load the complete file into memory.

  // reserve memory for file and load data
  try
    SetLength(SEQBuffer, FileSize(SEQFile));
    {$I-}
    BlockRead(SEQFile, SEQBuffer[0], Length(SEQBuffer));
    {$I+}
    if IOResult <> 0 then
    begin
      WriteLn('  read error');

      SetLength(SEQBuffer, 0);
      SEQBuffer := nil;

      Exit
    end
  except
    on EOutOfMemory do
    begin
      WriteLn('  not enough memory to load file');
      Exit
    end
  end;

  BufIdx := 0;

  // is the buffer able to store an SEQ header?
  if Length(SEQBuffer) - BufIdx < SizeOf(TSEQHeader) then
  begin
    WriteLn('  not a valid SEQ file');

    SetLength(SEQBuffer, 0);
    SEQBuffer := nil;

    Exit
  end;

  // get header
  SEQHeader := @SEQBuffer[BufIdx];
  Inc(BufIdx, SizeOf(TSEQHeader));

  // is the buffer able to store all frame entries?
  if Length(SEQBuffer) - BufIdx <
    Longint(SEQHeader^.NumFrames) * SizeOf(TSEQFrame) then
  begin
    WriteLn('  not a valid SEQ file');

    SetLength(SEQBuffer, 0);
    SEQBuffer := nil;

    Exit
  end;

  WriteLn(Format('  %d frames', [SEQHeader^.NumFrames]));

  Dec(BufIdx, SEQFrameSize);

  // process frames
  for i := 1 to SEQHeader^.NumFrames do
  begin
    WriteLn(Format('  frame #%.4d', [i]));

    Inc(BufIdx, SEQFrameSize);

    if not DumpFrame(SEQBuffer, BufIdx, FrameType_SEQ, FrameMode_Single,
      Format('%s_#%.4d', [ExtractFileName(SEQName), i])) then
      WriteLn(Format('    %s', [FrameDec.LastError]))
  end;

  SetLength(SEQBuffer, 0);
  SEQBuffer := nil
end;

procedure ScanFiles(FileParam: AnsiString);
var
  Dir,
  InName   : AnsiString;
  InFile   : file;
  SearchRec: TSearchRec;
begin
  // get the directory information from FileParam
  Dir := ExtractFilePath(FileParam);

  // search for all files matching FileParam
  if FindFirst(FileParam, faAnyFile - faDirectory, SearchRec) = 0 then
  begin
    repeat
      InName := Dir + SearchRec.Name;

      WriteLn(InName);

      // if this file is accessible, analyze it
      if OpenInFile(InFile, InName) then
      begin
        Process(InFile, InName);
        Close(InFile)
      end
      else
        WriteLn('  not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    WriteLn(Format('No matches for "%s"', [FileParam]));

  FindClose(SearchRec)
end;


begin
  if ParamCount = 1 then
    ScanFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.');
end.

