{ File   : SND2WAV.dpr
  Author : Deniz Oezmen
  Created: 2007-01-14
  Changed: 2010-08-29

  The Last Express SND converter.
}
program SND2WAV;

{$APPTYPE CONSOLE}

{$R '..\res\version.res' '..\res\version.rc'}

uses
  SysUtils,
  IMAHeader in 'sound\IMAHeader.pas',
  SoundDec  in 'sound\SoundDec.pas';

const
  SNDBlockSize = 739;

type
  TSNDHeader = packed record
    Size     : Longint;
    NumBlocks: Word;
  end;

function OpenInFile(var InFile: file; const InName: AnsiString): Boolean;
begin
  FileMode := fmOpenRead;
  AssignFile(InFile, InName);

  {$I-}
  Reset(InFile, 1);
  {$I+}
  Result := (IOResult = 0)
end;

procedure Process(var SNDFile: file; SNDName: AnsiString);
var
  SNDBuffer: array of Byte;
  SNDHeader: TSNDHeader;
begin
  SetLength(SNDBuffer, FileSize(SNDFile));
  {$I-}
  BlockRead(SNDFile, SNDBuffer[0], Length(SNDBuffer));
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn('  read error');

    SetLength(SNDBuffer, 0);
    Exit
  end;

  if Length(SNDBuffer) < SizeOf(SNDHeader) then
  begin
    WriteLn('  not a valid SND file');

    SetLength(SNDBuffer, 0);
    Exit
  end;

  Move(SNDBuffer[0], SNDHeader, SizeOf(SNDHeader));

  if (SNDHeader.Size > Length(SNDBuffer)) or
    (SNDHeader.NumBlocks * SNDBlockSize > Length(SNDBuffer)) then
    WriteLn('  size mismatch, output might be damaged');

  ClearSND;
  AddSNDBlocks(SNDBuffer);
  TruncSND(SNDHeader.NumBlocks);

  if not DumpSND(ExtractFileName(SNDName)) then
    WriteLn(Format('  %s' , [SoundDec.LastError]));

  SetLength(SNDBuffer, 0)
end;

procedure ScanFiles(FileParam: AnsiString);
var
  Dir,
  InName   : AnsiString;
  InFile   : file;
  SearchRec: TSearchRec;
begin
  // get the directory information from FileParam
  Dir := ExtractFilePath(FileParam);

  // search for all files matching FileParam
  if FindFirst(FileParam, faAnyFile - faDirectory, SearchRec) = 0 then
  begin
    repeat
      InName := Dir + SearchRec.Name;

      WriteLn(InName);

      // if this file is accessible, analyze it
      if OpenInFile(InFile, InName) then
      begin
        Process(InFile, InName);
        Close(InFile)
      end
      else
        WriteLn('  not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    WriteLn(Format('No matches for "%s"', [FileParam]));

  FindClose(SearchRec)
end;

begin
  if not SoundDec.Initialized then
  begin
    WriteLn(Format('Error: Could not initialize library %s.',
      [IMAHeader.DLLName]));
    Exit
  end;

  if ParamCount = 1 then
    ScanFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.')
end.

